#include <list>
#include "Problem.h"
#include "Solution.h"
#include "Control.h"
#include "Random.h"


#include <fstream.h>
#include <iostream.h>
using namespace std;


void tsOpt(Random* rnd, Control& control, Solution& initialSolution )
{

  vector<int> bestMove(3);
  vector<int> tabuMove(3);
  
  double largeDouble = 1.7E+308;
  
  //Read problem pointer from input solution.
  Problem* pp = initialSolution.getProblem();
  
  //Initialize current best solution.
  Solution currSol(rnd, control, pp);
  currSol.copySolution(initialSolution);

  //Solution for testing false positive solution, which actually are not
  //the best neighbouring solutions.
  Solution testSol(rnd,control,pp);
  
  //The cost of the true best solution.
  double trueCostBest = currSol.computeExpectedCost();
  
  //The possibly approximated costs of the neighbour solution.
  double bestNeighborCost;
  double neighborCost;


  //tabu list
  list< vector<int> > tabulist;

  int stringLen = 3;//1;
  
  
  while( control.timeLeft() )
    {//while(1)

      if ( stringLen < 1 ) stringLen =3;
      unsigned int tabulength;
      tabulength =
	(unsigned int)( (pp->numberOfCustomers -(stringLen +1) ) * 
			(0.8 + rnd->next() * 0.2 ) );
      
      //cout << "tabulen " << tabulength << endl;//// 
      
      bestNeighborCost = largeDouble;
      bool flagNewBestMove = false;
      
      bool bestMoveIsTabu=false;////////////
      
      if( currSol.firstMove(stringLen) )
	flagNewBestMove = false;
      bestMoveIsTabu = false;//////////

      vector<int> currMove(3);      
      do
	{
	  //Compute the neighbour cost by difference (possibly approximated) with the
	  //current solution cost.
	  neighborCost = currSol.computeMoveValue();
	  // if (neighborCost < bestNeighborCost) bestNeighborCost = neighborCost;///leo
	  
	  //currMove has customers *positions*.
	  currMove = currSol.getMove();	    
	  
	  //Check if the move is tabu. In tabu list there are customers labels.
	  bool tabu = false;
	  for( list< vector<int> >::iterator i =
		 tabulist.begin(); i != tabulist.end(); i++ )
	    {
	      if( ((*i)[0] == currSol[currMove[0]+1]) &&
		  ((*i)[1] ==  currMove[1]) && 
		  ((*i)[2] == currSol[currMove[2]]) ) {  

		tabu = true;
		//cout << "current_move_is_tabu" << endl;////
		break;
	      }
	    }
	  
	  //Update the best move for this solution.  
	  int pr = (int)(rnd->next() * 99);
	  if( (!tabu && pr<80) || (pr<30) )
	    //if(tabu == true) cout << "nucleus for tabu move" << endl;/////
	  //{ //Nucleus.
	      if( (neighborCost < bestNeighborCost) && 
		  ( !tabu || neighborCost < trueCostBest))
		{
		  //Put customers *positions* in bestMove.
		  bestMove[0] = currMove[0];
		  bestMove[1] = currMove[1];
		  bestMove[2] = currMove[2];
		  bestNeighborCost = neighborCost;
		  flagNewBestMove = true;
		  if(tabu) bestMoveIsTabu = true;
		  
		  
		}//end if
	  //}//end if 

          /*else {
            cout << "no nucleus" << endl;////
            if(tabu ) cout << "tabu but not best so far" << endl;////
            else cout << " not tabu but not neighbout best " << endl;
          }*/
	}while(currSol.nextMove(stringLen));
      
      
      //If there is at least one move.
      if(flagNewBestMove == true)
      {
	//Testing for false positive.	
	testSol.copySolution(currSol);
	testSol.setMove(bestMove);
	testSol.shift();
	double improvement = testSol.expectedCost-currSol.expectedCost;////
	//double improvement = testSol.expectedCost-bestNeighborCost;//// too restrictive!
	//if(improvement > 0.0) cout << "false_positive " << improvement << endl;///
	if(improvement <= 0.0)
	  {
	    currSol.setMove(bestMove);
	    currSol.shift();
	    //cout << currSol.expectedCost << endl;////
	    
	
	    //  else cout << "true_positive " << improvement << endl;////
	    //cout << "falseBestNeigh new_value " << improvement<<" " << currSol.expectedCost << endl;////  
	    
	    
	    //If the new current solution is really the new best, update the initial solution.
	    if ( currSol.expectedCost < trueCostBest )
	      {
		initialSolution.copySolution(currSol);
		trueCostBest = currSol.expectedCost;
		control.setCurrentCost( trueCostBest );
		
		// cout <<  currSol.expectedCost  << endl;///
		stringLen=3;
		//stringLen = stringLen%4;
	      }
	  }
	else	 stringLen--;
	
	//if(bestMoveIsTabu) cout << "bestMoveIsTabu "<< endl;///
	
        //Set the tabu moves according to the bestMove, and put them in tabu list.
	tabuMove[2] = currSol[bestMove[2]];
	for(int l=0; l<1; l++){
	  if(bestMove[0] > l){
	    tabuMove[0]= currSol[bestMove[0]-l];
	    tabuMove[1]= l+1;
	    tabulist.push_back( tabuMove );
	  }
	}

	tabuMove[0] = currSol[bestMove[0]+bestMove[1]+1];
	tabuMove[2] = currSol[bestMove[0]+bestMove[1]];
	for(int k = 1; k<2; k++){
	  tabuMove[1] = k;
	  tabulist.push_back( tabuMove );
	}
	
      }
      //else cout << "no new best_move, keeping same solution..." << endl;/////
            
      
      //tabulist.push_back( bestMove );
      while( tabulist.size() > tabulength ) 	tabulist.pop_front();
      
    }//end while(1)
}

	
void tsTspOpt(Random* rnd, Control& control, Solution& initialSolution )
{

  vector<int> bestMove(3);
  vector<int> tabuMove(3);
  
  double largeDouble = 1.7E+308;
  
  //Read problem pointer from input solution.
  Problem* pp = initialSolution.getProblem();
  
  //Initialize current best solution.
  Solution currSol(rnd, control, pp);
  currSol.copySolution(initialSolution);

  //Solution for testing false positive solution, which actually are not
  //the best neighbouring solutions.
  Solution testSol(rnd,control,pp);
  
  //The cost of the true best solution.
  double trueCostBest = currSol.computeExpectedCost();
  double trueLenBest = currSol.computeTourLength();  

  //The possibly approximated costs of the neighbour solution.
  double bestNeighborCost;
  double neighborCost;


  //tabu list
  list< vector<int> > tabulist;

  int stringLen = 3;//1;
  
  
  while( control.timeLeft() )
    {//while(1)

      if ( stringLen < 1 ) stringLen =3;
      unsigned int tabulength;
      tabulength =
	(unsigned int)( (pp->numberOfCustomers -(stringLen +1) ) * 
			(0.8 + rnd->next() * 0.2 ) );
      
      //cout << "tabulen " << tabulength << endl;//// 
      
      bestNeighborCost = largeDouble;
      bool flagNewBestMove = false;
      
      bool bestMoveIsTabu=false;////////////
      
      if( currSol.firstMove(stringLen) )
	flagNewBestMove = false;
      bestMoveIsTabu = false;//////////

      vector<int> currMove(3);      
      do
	{
	  //Compute the neighbour cost by difference (possibly approximated) with the
	  //current solution cost.
	  double len = currSol.computeTourLength();
	  neighborCost = currSol.computeMoveValueTSP(len);
	  //	  cout << "neighborCostLen " << neighborCost << endl;/////
	  // if (neighborCost < bestNeighborCost) bestNeighborCost = neighborCost;///leo
	  
	  //currMove has customers *positions*.
	  currMove = currSol.getMove();	    
	  
	  //Check if the move is tabu. In tabu list there are customers labels.
	  bool tabu = false;
	  for( list< vector<int> >::iterator i =
		 tabulist.begin(); i != tabulist.end(); i++ )
	    {
	      if( ((*i)[0] == currSol[currMove[0]+1]) &&
		  ((*i)[1] ==  currMove[1]) && 
		  ((*i)[2] == currSol[currMove[2]]) ) {  

		tabu = true;
		//cout << "current_move_is_tabu" << endl;////
		break;
	      }
	    }
	  
	  //Update the best move for this solution.  
	  int pr = (int)(rnd->next() * 99);
	  if( (!tabu && pr<80) || (pr<30) )
	    //if(tabu == true)
	    //	    cout << "nucleus " << endl;/////
	  //{ //Nucleus.
	      if( (neighborCost < bestNeighborCost) && 
		  ( !tabu || neighborCost < trueLenBest))
		{
		  //Put customers *positions* in bestMove.
		  bestMove[0] = currMove[0];
		  bestMove[1] = currMove[1];
		  bestMove[2] = currMove[2];
		  bestNeighborCost = neighborCost;
		  flagNewBestMove = true;
		  if(tabu) bestMoveIsTabu = true;
		  
		  
		}//end if
	  //}//end if 

          /*else {
            cout << "no nucleus" << endl;////
            if(tabu ) cout << "tabu but not best so far" << endl;////
            else cout << " not tabu but not neighbout best " << endl;
          }*/
	}while(currSol.nextMove(stringLen));
      
      
      //If there is at least one move.
      if(flagNewBestMove == true)
      {
	//	cout << "there is best move " << endl;/////
	//Testing for false positive.
	
	testSol.copySolution(currSol);
	testSol.setMove(bestMove);
	testSol.shift();
	double improvement = testSol.expectedCost-currSol.expectedCost;////
	//double improvement = testSol.expectedCost-bestNeighborCost;//// too restrictive!
	//if(improvement > 0.0) cout << "false_positive " << improvement << endl;///
	if(improvement <= 0.0)
	{
	    currSol.setMove(bestMove);
	    currSol.shiftTSP();
	    //cout << currSol.computeExpectedCost() << endl;////
	
	
	    //  else cout << "true_positive " << improvement << endl;////
	    //cout << "falseBestNeigh new_value " << improvement<<" " << currSol.expectedCost << endl;////  
	    
	    
	    //If the new current solution is really the new best, update the initial solution.
	    if ( currSol.computeExpectedCost() < trueCostBest )
	      {
		initialSolution.copySolution(currSol);
		trueCostBest = currSol.expectedCost;
		trueLenBest = currSol.computeTourLength();
		control.setCurrentCost( trueCostBest );
		
		// cout <<  currSol.expectedCost  << endl;///
		stringLen=3;
		//stringLen = stringLen%4;
	      }
	}
	else	 stringLen--;
	
	//if(bestMoveIsTabu) cout << "bestMoveIsTabu "<< endl;///

        //Set the tabu moves according to the bestMove, and put them in tabu list.
	tabuMove[2] = currSol[bestMove[2]];
	for(int l=0; l<1; l++){
	  if(bestMove[0] > l){
	    tabuMove[0]= currSol[bestMove[0]-l];
	    tabuMove[1]= l+1;
	    tabulist.push_back( tabuMove );
	  }
	}

	tabuMove[0] = currSol[bestMove[0]+bestMove[1]+1];
	tabuMove[2] = currSol[bestMove[0]+bestMove[1]];
	for(int k = 1; k<2; k++){
	  tabuMove[1] = k;
	  tabulist.push_back( tabuMove );
	}
	
      }
      else cout << "no new best_move, keeping same solution..." << endl;/////
            
      
      //tabulist.push_back( bestMove );
      while( tabulist.size() > tabulength ) 	tabulist.pop_front();
      
    }//end while(1)
}

	
	
